package com.nokia.dingtalk_api.pojos.po;

import cn.hutool.core.util.IdUtil;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.nokia.dingtalk_api.pojos.dto.cms.AddAppTaskDto;
import com.nokia.dingtalk_api.pojos.dto.cms.UpdateAppTaskDto;
import com.nokia.dingtalk_api.pojos.enums.AppTaskStatusEnum;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.io.Serial;
import java.io.Serializable;
import java.time.LocalDateTime;

/**
 * <p>
 * 应用定时任务
 * </p>
 *
 */
@AllArgsConstructor
@NoArgsConstructor
@Data
@TableName("dingtalk.app_task")
public class AppTaskPo implements Serializable {

    @Serial
    private static final long serialVersionUID = 1L;

    /**
     * 任务id
     */
    @TableId
    private String taskId;

    /**
     * 任务名称
     */
    private String taskName;

    /**
     * 任务描述
     */
    private String description;

    /**
     * 任务状态，0已停止1运行中2已删除
     */
    private Integer status;

    /**
     * cron表达式
     */
    private String cron;

    /**
     * 应用编码
     */
    private String appId;

    /**
     * 机器人编码
     */
    private String robotCode;

    /**
     * 会话类型：1：单聊，2：群聊
     */
    private Integer conversationType;

    /**
     * 群id
     */
    private String openConversationId;

    /**
     * 接收机器人消息的用户的手机号列表
     */
    private String phones;

    /**
     * 接收机器人消息的用户的userId列表
     */
    private String userIds;

    /**
     * sftp id
     */
    private String sftpId;

    /**
     * 数据文件夹
     */
    private String dataDir;

    /**
     * 文件名匹配正则表达式
     */
    private String filePattern;

    /**
     * 创建时间
     */
    private LocalDateTime createTime;

    /**
     * 更新时间
     */
    private LocalDateTime updateTime;

    public AppTaskPo(AddAppTaskDto dto) {
        LocalDateTime now = LocalDateTime.now();
        this.taskId = IdUtil.simpleUUID();
        this.taskName = dto.getTaskName();
        this.description = dto.getDescription();
        this.status = AppTaskStatusEnum.RUNNING.value;
        this.cron = dto.getCron();
        this.appId = dto.getAppId();
        this.robotCode = dto.getRobotCode();
        this.conversationType = dto.getConversationType();
        this.openConversationId = dto.getOpenConversationId();
        this.phones = dto.getPhones();
        this.userIds = dto.getUserIds();
        this.sftpId = dto.getSftpId();
        this.dataDir = dto.getDataDir();
        this.filePattern = dto.getFilePattern();
        this.createTime = now;
        this.updateTime = now;
    }

    public AppTaskPo(UpdateAppTaskDto dto) {
        LocalDateTime now = LocalDateTime.now();
        this.taskId = dto.getTaskId();
        this.taskName = dto.getTaskName();
        this.description = dto.getDescription();
        this.cron = dto.getCron();
        this.appId = dto.getAppId();
        this.robotCode = dto.getRobotCode();
        this.conversationType = dto.getConversationType();
        this.openConversationId = dto.getOpenConversationId();
        this.phones = dto.getPhones();
        this.userIds = dto.getUserIds();
        this.sftpId = dto.getSftpId();
        this.dataDir = dto.getDataDir();
        this.filePattern = dto.getFilePattern();
        this.updateTime = now;
    }
}
